# Generated by Django 2.0.2 on 2018-05-15 07:49

from django.db import migrations, models
import shard_static.models


class Migration(migrations.Migration):

    dependencies = [
        ('shard_static', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='staticsyncstatus',
            name='last_modified',
            field=models.DateTimeField(default=shard_static.models.get_first_day, verbose_name='Last Modified'),
        ),
    ]
