# Generated by Django 2.0.2 on 2018-05-02 11:19

from django.db import migrations, models
import shard.fields
import shard.mixins
import shard.strategy.id_generation.table
import shard_static.mixins


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='IdGenerateTestModel',
            fields=[
                ('id', shard.fields.TableStrategyPkField(primary_key=True, serialize=False, source_model='tests.TestIds', strategy=shard.strategy.id_generation.table.TableStrategy(source_model='tests.TestIds'))),
            ],
        ),
        migrations.CreateModel(
            name='NormalModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='ShardModelA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_id', models.IntegerField(verbose_name='유저 idx')),
                ('text', models.CharField(blank=True, max_length=64, verbose_name='더미 텍스트')),
            ],
            bases=(shard.mixins.ShardMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ShardModelB',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_id', models.IntegerField(verbose_name='유저 idx')),
            ],
            bases=(shard.mixins.ShardMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ShardStaticA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('last_modified', models.DateTimeField(auto_now=True, db_index=True, verbose_name='Last Modified')),
            ],
            options={
                'abstract': False,
            },
            bases=(shard_static.mixins.ShardStaticMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ShardStaticB',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('last_modified', models.DateTimeField(auto_now=True, db_index=True, verbose_name='Last Modified')),
            ],
            options={
                'abstract': False,
            },
            bases=(shard_static.mixins.ShardStaticMixin, models.Model),
        ),
        migrations.CreateModel(
            name='TestIds',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
            ],
            options={
                'abstract': False,
            },
            bases=(shard.mixins.SpecificDatabaseMixin, models.Model),
        ),
    ]
